import requests
from typing import Union
from .utils import __append_parameter__, __parse_parameter__
from .utils import CIVITAI_V1_URL as CIVITAI_BASE_URL

class Image():
    def __init__(self, jsondata) -> None:
        self.data = jsondata
        
    @property
    def id(self):
        return self.data["id"]
    
    @property
    def url(self):
        return self.data["url"]
    
    @property
    def hash(self):
        return self.data["hash"]
    
    @property
    def width(self):
        return self.data["width"]
    
    @property
    def height(self):
        return self.data["height"]
    
    @property
    def nsfw(self):
        return self.data["nsfw"]
    
    @property
    def nsfwLevel(self):
        return self.data["nsfwLevel"]
    
    @property
    def createdAt(self):
        return self.data["createdAt"]
    
    @property
    def postId(self):
        return self.data["postId"]
    
    @property
    def cryCount(self):
        return self.data["stats"]["cryCount"]
    
    @property
    def laughCount(self):
        return self.data["stats"]["laughCount"]
    
    @property
    def likeCount(self):
        return self.data["stats"]["likeCount"]
    
    @property
    def dislikeCount(self):
        return self.data["stats"]["dislikeCount"]
    
    @property
    def heartCount(self):
        return self.data["stats"]["heartCount"]
    
    @property
    def commentCount(self):
        return self.data["stats"]["commentCount"]  
    
    @property
    def meta(self):
        return self.data["meta"]    
    
    @property
    def username(self):
        return self.data["username"]
    
    @property
    def prompt(self):
        if self.meta is None: return None
        return self.meta["prompt"]
    
    
class ImagesResponse():
    def __init__(self, response) -> None:
        self.data = response
        self.items = [Image(i) for i in response["items"]]
    
    @property
    def nextCursor(self):
        return self.data["metadata"]["nextCursor"]
    
    @property
    def currentPage(self):
        return self.data["metadata"]["currentPage"]
    
    @property
    def pageSize(self):
        return self.data["metadata"]["pageSize"]
    
    @property
    def totalPages(self):
        return self.data["metadata"]["totalPages"]
    
    @property
    def nextPage(self):
        return self.data["metadata"]["nextPage"]
    
    
def get(
           limit:         int=None, 
           postId :       int=None, 
           modelId :      int=None, 
           modelVersionId:int=None, 
           username:      str=None, 
           nsfw:          Union[bool,str,None]=None, 
           sort:          str=None, 
           period:        str=None, 
           page:          int=None):
    params = [
    __parse_parameter__(limit=limit),
    __parse_parameter__(postId=postId),
    __parse_parameter__(modelId=modelId),
    __parse_parameter__(modelVersionId=modelVersionId),
    __parse_parameter__(username=username),
    __parse_parameter__(nsfw=nsfw),
    __parse_parameter__(sort=sort),
    __parse_parameter__(period=period),
    __parse_parameter__(page=page),
    ]    
    url = f'{CIVITAI_BASE_URL}/images{__append_parameter__(params)}'
    x = requests.get(url)
    return ImagesResponse(x.json())


import requests
from .utils import __append_parameter__, __parse_parameter__
from .utils import CIVITAI_V1_URL as CIVITAI_BASE_URL

class Creator():
    def __init__(self, jsondata) -> None:
        self.data = jsondata
        
    @property
    def username(self):
        return self.data["username"]
    
    @property
    def modelCount(self):
        return self.data["modelCount"]
    
    @property
    def link(self):
        return self.data["link"]
    
    @property
    def image(self):
        return self.data["image"]

class CreatorResponse():
    def __init__(self, response) -> None:
        self.data = response
        self.items = [Creator(i) for i in response["items"]]
    
    @property
    def totalItems(self):
        return self.data["metadata"]["totalItems"]
    
    @property
    def currentPage(self):
        return self.data["metadata"]["currentPage"]
    
    @property
    def pageSize(self):
        return self.data["metadata"]["pageSize"]
    
    @property
    def totalPages(self):
        return self.data["metadata"]["totalPages"]
    
    @property
    def nextPage(self):
        return self.data["metadata"]["nextPage"]
    
    
def get(limit:int=None, page:int=None, query:str=None):
    set_limit = __parse_parameter__(limit=limit)
    set_page  = __parse_parameter__(page=page)
    set_query = __parse_parameter__(query=query)
    url = f'{CIVITAI_BASE_URL}/creators{__append_parameter__([set_limit, set_page, set_query])}'
    x = requests.get(url)
    return CreatorResponse(x.json())


import requests
from .utils import __append_parameter__, __parse_parameter__
from .utils import CIVITAI_V1_URL as CIVITAI_BASE_URL

class ModelVersion:
    def __init__(self, data) -> None:
        self.data = data
    @property
    def id(self):
        return self.data["id"]
    @property
    def modelId(self):
        return self.data["modelId"]
    @property
    def name(self):
        return self.data["name"]
    @property
    def createdAt(self):
        return self.data["createdAt"]
    @property
    def updatedAt(self):
        return self.data["updatedAt"]
    @property
    def publishedAt(self):
        return self.data["publishedAt"]
    @property
    def trainedWords(self):
        return self.data["trainedWords"]
    @property
    def trainingStatus(self):
        return self.data["trainingStatus"]
    @property
    def trainingDetails(self):
        return self.data["trainingDetails"]
    @property
    def baseModel(self):
        return self.data["baseModel"]
    @property
    def baseModelType(self):
        return self.data["baseModelType"]
    @property
    def earlyAccessTimeFrame(self):
        return self.data["earlyAccessTimeFrame"]
    @property
    def description(self):
        return self.data["description"]
    @property
    def vaeId(self):
        return self.data["vaeId"]
    @property
    def stats(self):
        return self.data["stats"]
    @property
    def files(self):
        return self.data["files"]
    @property
    def images(self):
        return self.data["images"]
    @property
    def downloadUrl(self):
        return self.data["downloadUrl"]

class Model():
    def __init__(self, jsondata) -> None:
        self.data = jsondata
        self.modelVersions = [ModelVersion(mv) for mv in self.data["modelVersions"]]
    @property
    def id(self):
        return self.data["id"]
    
    @property
    def name(self):
        return self.data["name"]
    
    @property
    def description(self):
        return self.data["description"]
    
    @property
    def type(self):
        return self.data["type"]
    
    @property
    def nsfw(self):
        return self.data["nsfw"]
    
    @property
    def tags(self):
        return self.data["tags"]
    
    @property
    def mode(self):
        return self.data["mode"]
    
    @property
    def creator_username(self):
        return self.data["creator"]["username"]
    
    @property
    def creator_image(self):
        return self.data["creator"]["image"]
    
    @property
    def downloadCount(self):
        return self.data["stats"]["downloadCount"]
    
    @property
    def favoriteCount(self):
        return self.data["stats"]["favoriteCount"]
    
    @property
    def commentCount(self):
        return self.data["stats"]["commentCount"]
    
    @property
    def ratingCount(self):
        return self.data["stats"]["ratingCount"]
    
    @property
    def rating(self):
        return self.data["stats"]["rating"]
    
class ModelResponse():
    def __init__(self, response) -> None:
        self.data = response
        self.items = [Model(i) for i in response["items"]]
    
    @property
    def nextCursor(self):
        return self.data["metadata"]["nextCursor"]
    
    @property
    def currentPage(self):
        return self.data["metadata"]["currentPage"]
    
    @property
    def pageSize(self):
        return self.data["metadata"]["pageSize"]
    
    @property
    def totalPages(self):
        return self.data["metadata"]["totalPages"]
    
    @property
    def nextPage(self):
        return self.data["metadata"]["nextPage"]
    
def get(CIVITAI_BASE_URL:           str,
           limit:                   int=None, 
           page:                    int=None, 
           query:                   str=None, 
           tag:                     str=None, 
           username:                str=None, 
           types:                   str=None, 
           sort:                    str=None, 
           period:                  str=None, 
           rating:                  int=None,
           favorites:               bool=None,
           hidden:                  bool=None,
           primaryFileOnly:         bool=None,
           allowNoCredit:           bool=None,
           allowDerivatives:        bool=None,
           allowDifferentLicenses:  bool=None,
           allowCommercialUse:      bool=None,
           nsfw:                    bool=None,
           ):
    params = [
    __parse_parameter__(limit=limit),
    __parse_parameter__(page=page),
    __parse_parameter__(query=query),
    __parse_parameter__(tag=tag),
    __parse_parameter__(username=username),
    __parse_parameter__(types=types),
    __parse_parameter__(sort=sort),
    __parse_parameter__(period=period),
    __parse_parameter__(rating=rating),
    __parse_parameter__(favorites=favorites),
    __parse_parameter__(hidden=hidden),
    __parse_parameter__(primaryFileOnly=primaryFileOnly),
    __parse_parameter__(allowNoCredit=allowNoCredit),
    __parse_parameter__(allowDerivatives=allowDerivatives),
    __parse_parameter__(allowDifferentLicenses=allowDifferentLicenses),
    __parse_parameter__(allowCommercialUse=allowCommercialUse),
    __parse_parameter__(nsfw=nsfw),
    ]    
    url = f'{CIVITAI_BASE_URL}/models{__append_parameter__(params)}'
    x = requests.get(url)
    return ModelResponse(x.json())

def get_model(modelId: str):
    url = f"{CIVITAI_BASE_URL}/models/{modelId}"
    x = requests.get(url)
    return Model(x.json())

def get_by_modelVersion(modelVersionId: str):
    url = f"{CIVITAI_BASE_URL}/model-versions/{modelVersionId}"
    x = requests.get(url)
    return ModelVersion(x.json())

def get_by_hash(hash: str):
    url = f"{CIVITAI_BASE_URL}/model-versions/by-hash/{hash}"
    x = requests.get(url)
    return ModelVersion(x.json())



CIVITAI_V1_URL = "https://civitai.com/api/v1"

def __parse_parameter__(**kwargs):
    [(key, value)] = kwargs.items()
    return "" if value is None else f"{key}={value}"

def __append_parameter__(parameters=[]):
    return "?" + "&".join([p for p in parameters if p])